// ===========================
// CLASS BaseEmitter				
// ===========================

var ParticleEmitter = function(position, settings) {
	this.particles = [];
	this.maxParticles = 80;

	this.position = {x: position.x, y: position.y};
	
	this.remove = false;
	this.age = 0;
	
	this.angle = {min: settings.angle.min, max: settings.angle.max};
	this.shape = true;

	this.color = typeof settings.color == "undefined" ? colors.red : settings.color;
	
	this.lifespanParticle = {min: settings.lifespanParticle.min, max: settings.lifespanParticle.max};
	this.lifespanEmitter = getRandomInt(settings.lifespanEmitter.min,settings.lifespanEmitter.max);
	this.sizeParticle = typeof settings.sizeParticle == "undefined" ? 0 : settings.sizeParticle;
	this.speed = {min: settings.speed.min, max: settings.speed.max};
	this.gravity = {x: settings.gravity.x, y: settings.gravity.y};
	this.friction = typeof settings.friction == "undefined" ? 1.0 : settings.friction;
};

ParticleEmitter.prototype.update = function(timeElapsed) {	
	while(this.particles.length < this.maxParticles) 
		this.generateParticle();
	
	this.age += timeElapsed;
	
	this.updateParticles(timeElapsed);
	this.renderParticles();
	this.removeDeadParticles();
};

ParticleEmitter.prototype.updateParticles = function(timeElapsed) {
	for(var i = 0; i < this.particles.length; i++) {
		this.particles[i].update(timeElapsed);
		if(!this.particles[i].isAlive()) {
			this.particles[i].remove = true;
		}
	}
}

ParticleEmitter.prototype.render = function() {
	this.renderParticles();
};

ParticleEmitter.prototype.renderParticles = function() {
	for(var i = 0; i < this.particles.length; i++) {
		this.particles[i].render();
	}
}

ParticleEmitter.prototype.generateParticle = function() {	
	var angle = getRandomInt(this.angle.min, this.angle.max);
	var speed = getRandomInt(this.speed.min, this.speed.max);
	
	if(this.lifespanParticle.min == 0 && this.lifespanParticle.max == 0)
		var lifespan = this.lifespanEmitter;
	else
		var lifespan = getRandomInt(this.lifespanParticle.min,this.lifespanParticle.max);
		
	
	var settings = {
		x: this.position.x,
		y: this.position.y,
		size: this.sizeParticle,
		angle: angle,
		lifespan: lifespan,
		speed: speed,
		gravity: {x: this.gravity.x, y: this.gravity.y},
		friction: this.friction
	};
	
	var particle = new Particle(settings);
	particle.color = this.color;
	
	this.particles.push(particle);
};

ParticleEmitter.prototype.addParticle = function(particle) {
	this.particles.push(particle);
}

ParticleEmitter.prototype.removeParticle = function(particle) {
	var index = this.particles.indexOf(particle);
	
	if(index != -1)
		this.particles.splice(index, 1);
}

ParticleEmitter.prototype.removeAllParticles = function() {
	this.particles = [];
}

ParticleEmitter.prototype.removeDeadParticles = function() {
	for(var i = 0; i < this.particles.length; i++) {
		if(this.particles[i].remove) {
			this.removeParticle(this.particles[i]);
		}
	}
}

ParticleEmitter.prototype.isEmpty = function() {
	return this.particles.length == 0 ? true : false;
}

ParticleEmitter.prototype.isAlive = function() {
	if(this.lifespanEmitter == 0)
		return true;
	else 
		return this.age < this.lifespanEmitter ? true : false; 
}


// ===========================
// CLASS Particle				
// ===========================

var Particle = function(settings) {
	BaseObject.call(this, settings);
	this.lastPosition = {x: this.position.x, y: this.position.y};
	this.size = settings.size;
	this.color = {R: 255, G: 0, B: 0};
};

Particle.prototype = Object.create(BaseObject.prototype);
Particle.prototype.constructor = Particle;

Particle.prototype.update = function(timeElapsed) {
	
	if(this.age < this.lifespan || this.lifespan == 0) {
		this.lastPosition.x = this.position.x;
		this.lastPosition.y = this.position.y;
		
		this.speed *= this.friction;
		
		this.position.x += Math.cos(degreesToRadians(this.angle)) * this.speed + this.gravity.x;
		this.position.y += -(Math.sin(degreesToRadians(this.angle)) * this.speed) + this.gravity.y;
			
		this.age += timeElapsed;
		
		var ageRatio = this.age / this.lifespan;
		this.alpha = 1.0-ageRatio;
	}
};

Particle.prototype.render = function() {
	var canvas = document.getElementById(configuration.canvasID);
	c = canvas.getContext('2d');
	
	c.beginPath();
	
	c.strokeStyle = "rgba("+ this.color.R +","+ this.color.G +","+ this.color.B +", "+this.alpha+")"; 
	c.lineWidth = this.size;
	c.moveTo(this.lastPosition.x,this.lastPosition.y);
	c.lineTo(this.position.x, this.position.y);
	c.stroke();
	
	c.closePath();
}




